import time
from experiment_scheduler.master.master import Master
import grpc
from concurrent import futures
from experiment_scheduler.master.grpc_master.master_pb2_grpc import add_MasterServicer_to_server, MasterStub
from experiment_scheduler.master.grpc_master.master_pb2 import ExperimentStatement, MasterTaskStatement, \
    MasterResponse
from experiment_scheduler.task_manager.grpc_task_manager.task_manager_pb2_grpc import add_TaskManagerServicer_to_server, \
    TaskManagerStub
from experiment_scheduler.task_manager.task_manager_server import TaskManagerServicer
from multiprocessing import Process,set_start_method


def turn_master_on():
    master = grpc.server(futures.ThreadPoolExecutor(max_workers=4))
    add_MasterServicer_to_server(Master(), master)
    master.add_insecure_port('[::]:50052')
    print("Turn master on for testing")
    master.start()
    master.wait_for_termination()


def turn_task_manager_on():
    task_manager = grpc.server(futures.ThreadPoolExecutor(max_workers=4))
    add_TaskManagerServicer_to_server(TaskManagerServicer(), task_manager)
    task_manager.add_insecure_port('[::]:50051')
    print("Turn task manager on for testing")
    task_manager.start()
    task_manager.wait_for_termination()


class MasterTester:
    def __init__(self):
        master_process = Process(target=turn_master_on, )
        master_process.start()
        task_manager_process = Process(target=turn_task_manager_on, )
        task_manager_process.start()
        time.sleep(1)
        self.task_manager_stub = TaskManagerStub(grpc.insecure_channel("localhost:50051"))
        self.master_stub = MasterStub(grpc.insecure_channel("localhost:50052"))

    def test_master(self):
        self._request_experiments_test()
        self._script_running_test()
        self._submitter_request_test()

    def _submitter_request_test(self):
        # from experiment_scheduler.submitter.exs import exs_init_master
        # exs_init_master()
        # [TODO] Set assert logic + Redefine test method
        pass

    def _script_running_test(self):
        master_task_statement = MasterTaskStatement()
        master_task_statement.command = "python test/master/example/test.py"
        master_task_statement.name = "run_script"
        master_task_statement.task_env["RUNNING_SCRIPT"] = "WORKING"
        protobuf = ExperimentStatement(name="test2", tasks=[master_task_statement])
        response = self.master_stub.request_experiments(protobuf)
        print(response.experiment_id)
        assert type(response.experiment_id) is str
        assert response.response is MasterResponse.ResponseStatus.SUCCESS

    def _request_experiments_test(self):
        master_task_statement = MasterTaskStatement()
        master_task_statement.command = "echo $a"
        master_task_statement.name = "name1"
        master_task_statement.task_env["a"] = "b"
        protobuf = ExperimentStatement(name="test1", tasks=[master_task_statement])
        response = self.master_stub.request_experiments(protobuf)
        assert type(response.experiment_id) is str
        assert response.response is MasterResponse.ResponseStatus.SUCCESS

    def run_unit_test(self):
        pass


if __name__ == "__main__":
    set_start_method("fork")
    tester = MasterTester()
    tester.run_unit_test()
    tester.test_master()



import grpc
from experiment_scheduler.task_manager.grpc_task_manager.task_manager_pb2_grpc import TaskManagerStub
from experiment_scheduler.task_manager.grpc_task_manager.task_manager_pb2 import TaskStatement, Task,\
    google_dot_protobuf_dot_empty__pb2
from multiprocessing import Manager
import threading
import time


class ProcessMonitor:
    """
    ProcessMonitor communicates with TaskManagers.
    Select decent TaskManager for new task.
    All commands to TaskManager from Master must use ProcessMonitor
    """
    def __init__(self, task_manager, master_pipe):
        print(f"PM Start For {task_manager}")
        self.task_manager = task_manager
        self.channel = grpc.insecure_channel(self.task_manager)
        self.stub = TaskManagerStub(self.channel)
        self.master_pipe = master_pipe
        self.proto_empty = google_dot_protobuf_dot_empty__pb2.Empty()
        # connection initialization

        self.task_list = dict()
        self.completed_task_list = dict()
        # task list initializaiton

        self.shared_var_manager = Manager()
        self.thread_queue = self.shared_var_manager.dict()
        self.thread_queue["is_healthy"] = False
        # shared variable initialization

        self.health_checker = threading.Thread(target=self._health_check, args=(self.thread_queue,))
        self.health_checker.start()
        # health_checking_thread_on

        # shared with master memory

    def _health_check(self, thread_queue, time_interval=5):
        while True:
            response = self.stub.health_check(self.proto_empty)
            if response:
                thread_queue["is_healthy"] = True
            else:
                thread_queue["is_healthy"] = False
            time.sleep(time_interval)
    # should run this code through a thread.

    def is_healthy(self):
        return self.thread_queue["is_healthy"]

    def _request_task_manager(self, command):
        """
        all direct request to task manager use this method
        :param protobuf:
        :param request_type:
            :return:
        """
        # if request_task_manager need multithreading, use asyncio
        # multiprocessing, threading, asyncio
        # unify all task manager communication here.
        # [TODO] need refactoring later
        cmd = command[0]
        ret = ""
        if cmd == "kill_task":
            ret = self.kill_task(command[1])
        elif cmd == "get_task_status":
            ret = self.get_task_status(command[1])
        elif cmd == "get_all_tasks":
            ret = self.get_all_tasks()
        elif cmd == "run_task":
            ret = self.run_task(command[1], command[2], command[3], command[4])
        self.master_pipe.send(ret)

    def run_task(self, gpu_idx, command, name, env):
        protobuf = TaskStatement(gpuidx=gpu_idx, command=command, name=name, task_env=env)
        response = self.stub.run_task(protobuf)
        task_id = response.task_id
        return task_id

    def kill_task(self, task_id):
        protobuf = Task(task_id=task_id)
        return self.stub.kill_task(protobuf)

    def get_task_status(self,task_id):
        protobuf = Task(task_id=task_id)
        return self.stub.get_task_status(protobuf)

    def get_all_tasks(self):
        protobuf = self.proto_empty
        return self.stub.get_all_tasks(protobuf)

    def get_task_log(self,task_id):
        protobuf = Task(task_id=task_id)
        return self.stub.get_task_log(protobuf)

    def start(self, time_interval=1):
        # 로직 구현
        while True:
            # lock between sender and receiver must be set later
            if self.master_pipe.poll():
                command = self.master_pipe.recv()
                self._request_task_manager(command)
            time.sleep(time_interval)




"""
Setting file for Common Experiment-Scheduler Value.
In package, these variables work like constant, but they need initial setting.
"""
import configparser
import os
import experiment_scheduler


def _set_user_config():
    """
    set cfg file object to USER_CONFIG
    :return: ConfigParser Object
    """
    parser = configparser.ConfigParser()
    config_path = os.path.join(DEFAULT_EXS_HOME, "experiment_scheduler.cfg")
    parser.read(config_path)
    print("Set configuration path to %s" % config_path)
    return parser


HEADER = "\n".join(
    [
        "┌────────────────────────────────────┐",
        "│ Welcome to Experiment-Scheduler !! │",
        "└────────────────────────────────────┘",
    ]
)
DEFAULT_EXS_HOME = os.getenv("EXS_HOME", experiment_scheduler.__path__[0])
USER_CONFIG = _set_user_config()



"""
[TODO] exs execute command Explanation

"""
import argparse
import ast
import os
import yaml
import grpc
from experiment_scheduler.common.settings import USER_CONFIG
from experiment_scheduler.master.grpc_master import master_pb2
from experiment_scheduler.master.grpc_master import master_pb2_grpc


def parse_args():
    """
    Parse file name option argument.
    - ex) if exs command includes "-f sample.yaml", return "sample.yaml"
    """

    parser = argparse.ArgumentParser(description="Execute exeperiments.")
    parser.add_argument("-f", "--file")
    return parser.parse_args()


def parse_input_file(parsed_yaml):
    """
    Parse yaml and change yaml shape to experiment statement shape for grpc
    """
    input = master_pb2.ExperimentStatement(
        name=parsed_yaml["name"],
        tasks=[
            master_pb2.MasterTaskStatement(
                command=task["cmd"], name=task["name"], task_env=os.environ.copy()
            )
            for task in parsed_yaml["tasks"]
        ],
    )
    return input


def main():
    """
    exs execute command calls this function.
    when this func called, open yaml with '-f' option and convert yaml shape to experiment statement shape for grpc
    """
    args = parse_args()
    file_path = args.file

    with open(file_path) as f:
        parsed_yaml = yaml.load(f, Loader=yaml.FullLoader)
    channel = grpc.insecure_channel(
        ast.literal_eval(USER_CONFIG.get("default", "master_address"))
    )
    stub = master_pb2_grpc.MasterStub(channel)

    request = parse_input_file(parsed_yaml)
    response = stub.request_experiments(request)

    if response.response == master_pb2.MasterResponse.ResponseStatus.SUCCESS:
        print("experiment id is", response.experiment_id)
    else:
        print("fail to request experiments")



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: master.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database

# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import empty_pb2 as google_dot_protobuf_dot_empty__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(
    b'\n\x0cmaster.proto\x1a\x1bgoogle/protobuf/empty.proto"H\n\x13\x45xperimentStatement\x12\x0c\n\x04name\x18\x01 \x01(\t\x12#\n\x05tasks\x18\x02 \x03(\x0b\x32\x14.MasterTaskStatement"\x99\x01\n\x13MasterTaskStatement\x12\x0f\n\x07\x63ommand\x18\x01 \x01(\t\x12\x0c\n\x04name\x18\x02 \x01(\t\x12\x33\n\x08task_env\x18\x04 \x03(\x0b\x32!.MasterTaskStatement.TaskEnvEntry\x1a.\n\x0cTaskEnvEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01"\x82\x01\n\x0eMasterResponse\x12\x15\n\rexperiment_id\x18\x01 \x01(\t\x12\x30\n\x08response\x18\x02 \x01(\x0e\x32\x1e.MasterResponse.ResponseStatus"\'\n\x0eResponseStatus\x12\x0b\n\x07SUCCESS\x10\x00\x12\x08\n\x04\x46\x41IL\x10\x01\x32\x92\x01\n\x06Master\x12>\n\x13request_experiments\x12\x14.ExperimentStatement\x1a\x0f.MasterResponse"\x00\x12H\n\x14halt_process_monitor\x12\x16.google.protobuf.Empty\x1a\x16.google.protobuf.Empty"\x00\x42\x06\xa2\x02\x03RTGb\x06proto3'
)


_EXPERIMENTSTATEMENT = DESCRIPTOR.message_types_by_name["ExperimentStatement"]
_MASTERTASKSTATEMENT = DESCRIPTOR.message_types_by_name["MasterTaskStatement"]
_MASTERTASKSTATEMENT_TASKENVENTRY = _MASTERTASKSTATEMENT.nested_types_by_name[
    "TaskEnvEntry"
]
_MASTERRESPONSE = DESCRIPTOR.message_types_by_name["MasterResponse"]
_MASTERRESPONSE_RESPONSESTATUS = _MASTERRESPONSE.enum_types_by_name["ResponseStatus"]
ExperimentStatement = _reflection.GeneratedProtocolMessageType(
    "ExperimentStatement",
    (_message.Message,),
    {
        "DESCRIPTOR": _EXPERIMENTSTATEMENT,
        "__module__": "master_pb2"
        # @@protoc_insertion_point(class_scope:ExperimentStatement)
    },
)
_sym_db.RegisterMessage(ExperimentStatement)

MasterTaskStatement = _reflection.GeneratedProtocolMessageType(
    "MasterTaskStatement",
    (_message.Message,),
    {
        "TaskEnvEntry": _reflection.GeneratedProtocolMessageType(
            "TaskEnvEntry",
            (_message.Message,),
            {
                "DESCRIPTOR": _MASTERTASKSTATEMENT_TASKENVENTRY,
                "__module__": "master_pb2"
                # @@protoc_insertion_point(class_scope:MasterTaskStatement.TaskEnvEntry)
            },
        ),
        "DESCRIPTOR": _MASTERTASKSTATEMENT,
        "__module__": "master_pb2"
        # @@protoc_insertion_point(class_scope:MasterTaskStatement)
    },
)
_sym_db.RegisterMessage(MasterTaskStatement)
_sym_db.RegisterMessage(MasterTaskStatement.TaskEnvEntry)

MasterResponse = _reflection.GeneratedProtocolMessageType(
    "MasterResponse",
    (_message.Message,),
    {
        "DESCRIPTOR": _MASTERRESPONSE,
        "__module__": "master_pb2"
        # @@protoc_insertion_point(class_scope:MasterResponse)
    },
)
_sym_db.RegisterMessage(MasterResponse)

_MASTER = DESCRIPTOR.services_by_name["Master"]
if _descriptor._USE_C_DESCRIPTORS == False:

    DESCRIPTOR._options = None
    DESCRIPTOR._serialized_options = b"\242\002\003RTG"
    _MASTERTASKSTATEMENT_TASKENVENTRY._options = None
    _MASTERTASKSTATEMENT_TASKENVENTRY._serialized_options = b"8\001"
    _EXPERIMENTSTATEMENT._serialized_start = 45
    _EXPERIMENTSTATEMENT._serialized_end = 117
    _MASTERTASKSTATEMENT._serialized_start = 120
    _MASTERTASKSTATEMENT._serialized_end = 273
    _MASTERTASKSTATEMENT_TASKENVENTRY._serialized_start = 227
    _MASTERTASKSTATEMENT_TASKENVENTRY._serialized_end = 273
    _MASTERRESPONSE._serialized_start = 276
    _MASTERRESPONSE._serialized_end = 406
    _MASTERRESPONSE_RESPONSESTATUS._serialized_start = 367
    _MASTERRESPONSE_RESPONSESTATUS._serialized_end = 406
    _MASTER._serialized_start = 409
    _MASTER._serialized_end = 555
# @@protoc_insertion_point(module_scope)



import os
import textwrap

pynvml = None
ALLOW_LEGACY_PYNVML = os.getenv("ALLOW_LEGACY_PYNVML", "")
ALLOW_LEGACY_PYNVML = ALLOW_LEGACY_PYNVML.lower() not in ('false', '0', '')

try:
    import pynvml
    if not (
        hasattr(pynvml, 'NVML_BRAND_NVIDIA_RTX') or
        hasattr(pynvml, 'nvmlDeviceGetComputeRunningProcesses_v2')
    ):
        raise RuntimeError("pynvml library is outdated.")
except (ImportError, SyntaxError, RuntimeError) as e:
    raise ImportError(textwrap.dedent(
        """\
        pynvml is missing or an outdated version is installed.
        Your pynvml installation: """ + repr(pynvml))) from e

__all__ = ['pynvml']


import psutil as psutil
import logging
from experiment_scheduler.resource_monitor.setting import pynvml as N


class responser(object):
    global_processes = {}

    def __init__(self):
        self.queued_gpu_idx = []

    @staticmethod
    def get_all_gpu_info():
        logger = logging.getLogger()
        N.nvmlInit()

        def get_gpu_info(handle):

            def get_process_info(target_process):
                process = {}
                if nv_process.pid not in responser.global_processes:
                    responser.global_processes[target_process.pid] = \
                        psutil.Process(pid=target_process.pid)
                process['pid'] = target_process.pid
                return process

            try:
                nv_comp_processes = \
                    N.nvmlDeviceGetComputeRunningProcesses(handle)
            except N.NVMLError as e:
                logger.info("compute_processes", e)
                nv_comp_processes = None
            try:
                nv_graphics_processes = \
                    N.nvmlDeviceGetGraphicsRunningProcesses(handle)
            except N.NVMLError as e:
                logger.info("graphics_processes", e)
                nv_graphics_processes = None

            if nv_comp_processes is None and nv_graphics_processes is None:
                processes = None

            else:
                processes = []
                nv_comp_processes = nv_comp_processes or []
                nv_graphics_processes = nv_graphics_processes or []
                seen_pids = set()
                for nv_process in nv_comp_processes + nv_graphics_processes:
                    if nv_process.pid in seen_pids:
                        continue
                    seen_pids.add(nv_process.pid)
                    process = get_process_info(nv_process)
                    processes.append(process)

            util_info = N.nvmlDeviceGetUtilizationRates(handle)

            return {'gpu-index': N.nvmlDeviceGetIndex(handle),
                    'processes': processes, 'available_util': (100 - util_info.gpu)}

        gpu_all_stat = []
        device_count = N.nvmlDeviceGetCount()

        for index in range(device_count):
            handle = N.nvmlDeviceGetHandleByIndex(index)
            info = get_gpu_info(handle)
            gpu_all_stat.append(info)

        return gpu_all_stat

    @staticmethod
    def get_max_free_gpu():
        gpu_all_stat = responser.get_all_gpu_info()
        max_free_gpu = {'gpu-index': -1, 'process': None, 'available_util': -1}
        for gpu in gpu_all_stat:
            if gpu.get('available_util') > max_free_gpu.get('available_util') and gpu.get('available_util') > 20:
                max_free_gpu = gpu
        return max_free_gpu.get('gpu-index')



"""
[TODO] exs command Explanation

"""

import argparse
import sys

from experiment_scheduler.submitter.execute import main as exs_execute
from experiment_scheduler.submitter.delete import main as exs_delete
from experiment_scheduler.submitter.edit import main as exs_edit
from experiment_scheduler.submitter.list import main as exs_list
from experiment_scheduler.submitter.status import main as exs_status
from experiment_scheduler.submitter.init_master import main as exs_init_master
from experiment_scheduler.submitter.init_task_manager import (
    main as exs_init_task_manager,
)


COMMAND_LIST = {
    "execute": exs_execute,
    "delete": exs_delete,
    "edit": exs_edit,
    "list": exs_list,
    "status": exs_status,
    "init_master": exs_init_master,
    "init_task_manager": exs_init_task_manager,
}


def parse_args():
    """
    Parse user's arguments.
    """

    parser = argparse.ArgumentParser(add_help=False)
    parser.add_argument("operation", choices=list(COMMAND_LIST.keys()))

    return parser.parse_known_args()[0]


def main():
    """
    Select and execute a function from the command list.
    """
    name = parse_args().operation
    del sys.argv[1]
    COMMAND_LIST[name]()


if __name__ == "__main__":
    main()



from experiment_scheduler.common import settings
import os
import subprocess
import time
import sys
# [TODO] daemon option


def main():
    exs_home = os.getenv("EXS_HOME")
    # [TODO] set master's path as constant in settings
    command = ([sys.executable, "-u", os.path.join(exs_home, "master/master.py")])
    print("Master is initiated. Please wait for a second...")
    task = subprocess.Popen(
        args=command,
        shell=False,
        stdout=subprocess.PIPE,
        stderr=subprocess.STDOUT
    )
    # [TODO] Add Address
    print("Master is now set")
    while True:
        output = task.stdout.readline().decode('utf-8')
        if len(output) > 0:
            print(output, end='')
        sys.stdout.flush()









# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc

from google.protobuf import empty_pb2 as google_dot_protobuf_dot_empty__pb2
from experiment_scheduler.master.grpc_master import master_pb2 as master__pb2


class MasterStub(object):
    """Interface exported by the server."""

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.request_experiments = channel.unary_unary(
            "/Master/request_experiments",
            request_serializer=master__pb2.ExperimentStatement.SerializeToString,
            response_deserializer=master__pb2.MasterResponse.FromString,
        )
        self.halt_process_monitor = channel.unary_unary(
            "/Master/halt_process_monitor",
            request_serializer=google_dot_protobuf_dot_empty__pb2.Empty.SerializeToString,
            response_deserializer=google_dot_protobuf_dot_empty__pb2.Empty.FromString,
        )


class MasterServicer(object):
    """Interface exported by the server."""

    def request_experiments(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def halt_process_monitor(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")


def add_MasterServicer_to_server(servicer, server):
    rpc_method_handlers = {
        "request_experiments": grpc.unary_unary_rpc_method_handler(
            servicer.request_experiments,
            request_deserializer=master__pb2.ExperimentStatement.FromString,
            response_serializer=master__pb2.MasterResponse.SerializeToString,
        ),
        "halt_process_monitor": grpc.unary_unary_rpc_method_handler(
            servicer.halt_process_monitor,
            request_deserializer=google_dot_protobuf_dot_empty__pb2.Empty.FromString,
            response_serializer=google_dot_protobuf_dot_empty__pb2.Empty.SerializeToString,
        ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
        "Master", rpc_method_handlers
    )
    server.add_generic_rpc_handlers((generic_handler,))


# This class is part of an EXPERIMENTAL API.
class Master(object):
    """Interface exported by the server."""

    @staticmethod
    def request_experiments(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/Master/request_experiments",
            master__pb2.ExperimentStatement.SerializeToString,
            master__pb2.MasterResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def halt_process_monitor(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/Master/halt_process_monitor",
            google_dot_protobuf_dot_empty__pb2.Empty.SerializeToString,
            google_dot_protobuf_dot_empty__pb2.Empty.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )



"""
[TODO] Master Explanation

"""
from concurrent import futures
from multiprocessing import Process, Pipe
import uuid
import time
import threading
import ast
import grpc
from experiment_scheduler.master.process_monitor import ProcessMonitor
from experiment_scheduler.master.grpc_master.master_pb2_grpc import (
    MasterServicer,
    add_MasterServicer_to_server,
    MasterStub,
)
from experiment_scheduler.master.grpc_master import master_pb2
from experiment_scheduler.common import settings
from experiment_scheduler.resource_monitor.monitor import responser
from experiment_scheduler.common.settings import USER_CONFIG

def get_task_managers():
    """
    [TODO] add docstring
    :return:
    """
    return ast.literal_eval(USER_CONFIG.get("default", "task_manager_address"))


class Master(MasterServicer):
    """
    Inherit GrpcServer to run Grpc Socket ( get request from submitter )
    Class that runs on server
    """

    def __init__(self):
        """
        Init GrpcServer.
        """
        # [Todo] Logging required
        # [TODO] need discussion about path and env vars
        self.queued_tasks = []
        self.master_pipes = dict()
        self.process_monitor_pipes = dict()
        self.task_managers_address = get_task_managers()
        self.process_monitor = self.create_process_monitor()
        self.runner = threading.Thread(target=self._execute_command, daemon=True)
        self.runner.start()

    def _execute_command(self, interval=1):
        """
        this thread_running_function periodically checks queued_task and available task_managers.
        If a task exists and available task manager exists, toss command to Process Monitor
        :param interval: time interval
        :return: None
        """
        while True:
            (available_task_managers, gpu_idx) = self.get_available_task_managers()
            if gpu_idx != -1:
                for _ in self.queued_tasks:
                    if len(available_task_managers) > 0:
                        task_manager = available_task_managers.pop(0)
                        self.execute_task(task_manager, gpu_idx)
                # [TODO] master pipe recv logic required here
                for task_manager, pipe in self.master_pipes.items():
                    if pipe.poll():
                        print(pipe.recv())  # need change later
            time.sleep(interval)

    def _run_process_monitor(  # pylint: disable=no-self-use
        self, task_manager_address, pipe
    ):
        """
        Create Process Monitor process per task manager
        :param task_manager_address:
        :param pipe:
        :return:
        """
        process_monitor = ProcessMonitor(task_manager_address, pipe)
        process_monitor.start()

    def _process_monitor_termintion(self):
        # [TODO] all process monitors should be halted if master stopped
        pass

    def create_process_monitor(self):
        """
        [TODO] add docstring
        :return:
        """
        print("create process monitor")  # print should be replaced as log later
        process_monitor_list = list()
        for task_manager in self.task_managers_address:
            (
                self.master_pipes[task_manager],
                self.process_monitor_pipes[task_manager],
            ) = Pipe()
            process_monitor = Process(
                target=self._run_process_monitor,
                args=(task_manager, self.process_monitor_pipes[task_manager]),
            )
            process_monitor_list.append(process_monitor)
            process_monitor.start()
        return process_monitor_list

    def select_task_manager(self, selected=-1):
        """
        Process Monitor automatically provide task that is able to run task
        :return:
        """
        # [TODO] need to set convention with resource monitor later
        return (
            self.task_managers_address[0]
            if selected < 0
            else self.task_managers_address[selected]
        )

    def request_experiments(self, request, context):
        """
        [TODO] add docstring
        :param request:
        :param context:
        :return:
        """
        experiment_id = request.name + "-" + str(uuid.uuid1())
        print(experiment_id)
        for task in request.tasks:
            self.queued_tasks.append(task)
        response_status = (
            master_pb2.MasterResponse.ResponseStatus  # pylint: disable=E1101
        )
        response = (
            response_status.SUCCESS
            if experiment_id is not None
            else response_status.FAIL
        )
        return master_pb2.MasterResponse(experiment_id=experiment_id, response=response)

    def halt_process_monitor(self, request, context):
        for process in self.process_monitor:
            process.terminate()
        return master_pb2.google_dot_protobuf_dot_empty__pb2.Empty()

    def delete_experiment(self, request, context):
        """
        delete all experiments registered in a group
        :param request:
        :param context:
        :return:
        """
        pass  # pylint: disable=unnecessary-pass

    def delete_experiments(self, request, context):
        """
        delete certain experiment
        :param request:
        :param context:
        :return:
        """
        pass  # pylint: disable=unnecessary-pass

    def check_task_manager_run_task_available(  # pylint: disable=unused-argument,no-self-use
        self, task_manager
    ):
        """
        [TODO] add docstring
        :param task_manager:
        :return:
        """
        # currently only returns True
        return True

    def get_available_task_managers(self):
        """
        [TODO] add docstring
        :return:
        """
        # currently only return first one
        available_task_managers = []
        for task_manager in self.task_managers_address[0]:
            if self.check_task_manager_run_task_available(task_manager):
                available_task_managers.append(task_manager)
        gpu_idx = responser.get_max_free_gpu()
        # return available_task_managers
        return tuple([self.task_managers_address[0], gpu_idx])

    def execute_task(self, task_manager, gpu_idx):
        """
        [TODO] add docstring
        :param task_manager:
        :param gpu_idx:
        :return:
        """
        prior_task = self.queued_tasks.pop(0)
        self.master_pipes[task_manager].send(
            [
                "run_task",
                gpu_idx,
                prior_task.command,
                prior_task.name,
                dict(prior_task.task_env),
            ]
        )

def halt_process_monitor():
    """
    kill process monitor before close master server
    process monitor can be closed through communication with master
    :return: None
    """
    stub = MasterStub(grpc.insecure_channel("localhost:50052"))
    empty = master_pb2.google_dot_protobuf_dot_empty__pb2.Empty()
    stub.halt_process_monitor(empty)


def serve():
    """
    Run Master Server with try, catch.
    If an anomaly action erupt, kill process monitor before close master object
    :return: None
    """
    
    with futures.ThreadPoolExecutor(max_workers=10) as pool:
        master = grpc.server(pool)
        print(settings.HEADER)
        master_address = ast.literal_eval(USER_CONFIG.get("default", "master_address"))
        print("set master server to %s" % master_address)
        add_MasterServicer_to_server(Master(), master)
        master.add_insecure_port(master_address)
        try:
            master.start()
            master.wait_for_termination()
        except KeyboardInterrupt as exception:
            print("keyboardInterrupt occurred \n %s", exception)
            print("halting master immediately...")
            halt_process_monitor()
        except Exception as error_case:  # pylint: disable=broad-except
            print("Error Occurred %s", error_case)
            print("halting master immediately...")
            halt_process_monitor()


if __name__ == "__main__":
    serve()



"""This file is in charge of server code run as daemon process."""

import logging
import os
import signal
import subprocess
import uuid
from concurrent import futures
from os import path as osp

import grpc

from experiment_scheduler.task_manager.grpc_task_manager import task_manager_pb2_grpc
from experiment_scheduler.task_manager.grpc_task_manager.task_manager_pb2 import (
    ServerStatus,
    TaskStatus,
    AllTasksStatus,
    TaskLog,
)

logger = logging.getLogger(__name__)


class TaskManagerServicer(task_manager_pb2_grpc.TaskManagerServicer):
    """Provides methods that implement functionality of task manager server."""

    # pylint: disable=no-member

    def __init__(self, log_dir=os.getcwd()):
        self.tasks = {}
        self.log_dir = log_dir

    def health_check(self, request, context):
        """Return current server status"""
        return ServerStatus(alive=True)

    def run_task(self, request, context):
        """run task based on request"""
        self._validate_task_statement(request)

        task_id = self._create_task_id(request.name)
        task = self._execute_subprocess(request, task_id)
        self._register_task(task_id, task)
        logger.info("%s is now running!", task_id)

        return TaskStatus(task_id=task_id, status=TaskStatus.Status.RUNNING)

    def _validate_task_statement(self, task_statement):
        if task_statement.gpuidx < 0:
            raise ValueError(
                f"GPU index should be positive. your value is {task_statement.gpuidx}"
            )
        if not task_statement.command:
            raise ValueError("Command shouldn't empty!")

    def _create_task_id(self, name):
        return name + "-" + uuid.uuid4().hex  # add random hash to make task_id

    def _execute_subprocess(self, task_statement, task_id: str):
        # pylint: disable=consider-using-with
        task_env = task_statement.task_env
        task_env["CUDA_VISIBLE_DEVICES"] = str(task_statement.gpuidx)
        log_file_path = osp.join(self.log_dir, f"{task_id}_log.txt")
        output_file = open(log_file_path, "w", encoding="utf-8")

        task = subprocess.Popen(
            args=task_statement.command,
            shell=True,
            env=task_env,
            stdout=output_file,
            stderr=subprocess.STDOUT,
        )

        return task

    def _register_task(self, task_id, task):
        self.tasks[task_id] = task

    def get_task_log(self, request, context):
        """
        Save an output of the requested task and return output file path.
        If status of the requeest task is Done, delete it from task manager.
        """
        target_process = self._get_task(request.task_id)
        if target_process is None:
            return TaskStatus(logfile_path="")

        log_file_path = osp.join(self.log_dir, f"{request.task_id}_log.txt")

        if self.tasks[request.task_id].poll() is not None:
            del self.tasks[request.task_id]

        return TaskLog(logfile_path=log_file_path)

    def kill_task(self, request, context):
        """Kill a requsted task if the task is running"""
        target_process = self._get_task(request.task_id)

        if target_process is None:
            return TaskStatus(
                task_id=request.task_id, status=TaskStatus.Status.NOTFOUND
            )
        sign = target_process.poll()

        if sign is not None:
            return TaskStatus(task_id=request.task_id, status=TaskStatus.Status.DONE)

        target_process.terminate()
        target_process.wait()
        logger.info("%s is killed!", request.task_id)
        return TaskStatus(task_id=request.task_id, status=TaskStatus.Status.KILLED)

    def get_task_status(self, request, context):
        """Get single requested task status"""
        target_process = self._get_task(request.task_id)

        if target_process is None:
            return TaskStatus(
                task_id=request.task_id, status=TaskStatus.Status.NOTFOUND
            )

        return self._wrap_by_grpc_task_status(request.task_id)

    def get_all_tasks(self, request, context):
        """Get all tasks managed by task manager"""
        all_tasks_status = AllTasksStatus()

        for task_id in self.tasks:
            all_tasks_status.task_status_array.append(
                self._wrap_by_grpc_task_status(task_id)
            )

        return all_tasks_status

    def _wrap_by_grpc_task_status(self, task_id):
        """Make task_manager_pb2.TaskStatus using return code of task."""
        target_process = self._get_task(task_id)

        if target_process is None:
            return TaskStatus(task_id=task_id, status=TaskStatus.Status.NOTFOUND)

        return_code = target_process.poll()
        if return_code == 0:
            return TaskStatus(task_id=task_id, status=TaskStatus.Status.DONE)
        if return_code is None:
            return TaskStatus(task_id=task_id, status=TaskStatus.Status.RUNNING)
        if return_code in [-signal.SIGTERM, return_code == -signal.SIGKILL]:
            return TaskStatus(task_id=task_id, status=TaskStatus.Status.KILLED)
        return TaskStatus(task_id=task_id, status=TaskStatus.Status.ABNORMAL)

    def _get_task(self, task_id):
        """Get a task instance if exists. if not, return None"""
        if task_id not in self.tasks:
            logger.warning("%s is not found in task_manager!", task_id)
            return None
        return self.tasks[task_id]


def serve():
    """run task manager server"""
    server = grpc.server(futures.ThreadPoolExecutor(max_workers=10))
    task_manager_pb2_grpc.add_TaskManagerServicer_to_server(
        TaskManagerServicer(), server
    )
    server.add_insecure_port("[::]:50051")
    server.start()
    server.wait_for_termination()


if __name__ == "__main__":
    serve()



from unittest import TestCase
from unittest.mock import Mock, patch
import uuid
from experiment_scheduler.master.master import Master
import experiment_scheduler


def mockGetTaskManagers():
    return ["test_network"]


class MockPipe:
    def __init__(self, *args, **kwargs):
        pass

    def send(self, *args, **kwargs):
        pass

    def poll(self, *args, **kwargs):
        pass

    def recv(self, *args, **kwargs):
        pass


def mockPipe():
    return (MockPipe(), MockPipe())


class TestRequest:
    name = "test_name"
    tasks = "test_tasks"

    def __init__(self, *args, **kwargs):
        pass


class MockResponser:
    def __init__(self, *args, **kwargs):
        pass

    def get_all_gpu_info(*args, **kwargs):
        return [1, 2, 3]

    def get_max_free_gpu(*args, **kwargs):
        return 1


class MockProcess:
    def __init__(self, *args, **kwargs):
        pass

    def start(self):
        pass


class MockProcessMonitor:
    def __init__(self, *args, **kwargs):
        pass

    def start(self):
        pass


class MockThread:
    def __init__(self, *args, **kwargs):
        pass

    def start(self):
        pass


class MockTask:
    name = "test"
    command = "task"
    task_env = {"test": "env"}

    def __init__(self, *args, **kwargs):
        pass


class TestMaster(TestCase):
    @patch.object(
        experiment_scheduler.master.master, "ProcessMonitor", MockProcessMonitor
    )
    @patch.object(experiment_scheduler.master.master, "Process", MockProcess)
    @patch.object(experiment_scheduler.master.master, "Pipe", mockPipe)
    @patch.object(
        experiment_scheduler.master.master, "get_task_managers", mockGetTaskManagers
    )
    @patch("threading.Thread", MockThread)
    def setUp(self):
        Master.get_task_managers = Mock(return_value=mockGetTaskManagers())
        self.master = Master()
        self.master.queued_tasks = [MockTask()]

    def tearDown(self):
        patch.stopall()

    @patch("time.sleep", side_effect=Exception)
    @patch.object(
        experiment_scheduler.resource_monitor.monitor, "responser", MockResponser
    )
    def test__execute_command(self, mock_time_sleep):
        # given
        self.master.execute_task = Mock()
        self.master.get_available_task_managers = Mock(
            return_value=(["test_network"], 1)
        )

        # when
        self.assertRaises(Exception, lambda: self.master._execute_command())

        # then
        self.master.execute_task.assert_called_with("test_network", 1)
        self.assertIsInstance(self.master.master_pipes["test_network"], MockPipe)

    @patch.object(
        experiment_scheduler.master.master, "ProcessMonitor", MockProcessMonitor
    )
    def test__run_process_monitor(self):
        # given
        MockProcessMonitor.start = Mock()

        # when
        self.master._run_process_monitor("test_tm_address", MockPipe())

        # then
        MockProcessMonitor.start.assert_called_once()

    def test__process_monitor_termintion(self):
        pass

    @patch.object(experiment_scheduler.master.master, "Process", MockProcess)
    @patch.object(experiment_scheduler.master.master, "Pipe", mockPipe)
    def test_create_process_monitor(self):
        # given
        self.master._run_process_monitor = Mock()

        # # when
        test_return_value = self.master.create_process_monitor()

        # then
        self.assertIsInstance(test_return_value, list)

    def test_get_task_managers(self):
        # when
        test_return_value = self.master.get_task_managers()

        # then
        self.assertEqual(test_return_value, ["test_network"])

    def test_select_task_manager(self):
        # when
        test_return_value = self.master.select_task_manager()

        # then
        self.assertEqual(test_return_value, "test_network")

    @patch.object(uuid, "uuid1", (lambda: 123))
    def test_request_experiments(self):
        # given
        test_request = TestRequest()

        # when
        test_return_value = self.master.request_experiments(test_request, "context")

        # then
        self.assertEqual(test_return_value.experiment_id, "test_name-123")
        self.assertEqual(test_return_value.response, 0)

    def test_delete_experiment(self):
        pass

    def test_delete_experiments(self):
        pass

    def test_check_task_manager_run_task_available(self):
        # when
        test_return_value = self.master.check_task_manager_run_task_available(
            "test_network"
        )

        # then
        self.assertEqual(test_return_value, True)

    @patch.object(experiment_scheduler.master.master, "responser", MockResponser)
    def test_get_available_task_managers(self):
        # when
        test_return_value = self.master.get_available_task_managers()

        # then
        self.assertEqual(test_return_value, ("test_network", 1))

    @patch.object(experiment_scheduler.master.master, "Pipe", mockPipe)
    def test_execute_task(self):
        # when
        self.master.execute_task(task_manager="test_network", gpu_idx=1)

        # then
        self.assertIsInstance(self.master.master_pipes["test_network"], MockPipe)



from unittest import TestCase
from unittest.mock import Mock, patch

import experiment_scheduler
from experiment_scheduler.master.process_monitor import ProcessMonitor


class MockPipe:
    def __init__(self, *args, **kwargs):
        pass

    def send(self, *args, **kwargs):
        pass

    def poll(self, *args, **kwargs):
        pass

    def recv(self, *args, **kwargs):
        pass

class Response:
    task_id = 'test_id'
    def __init__(self, *args, **kwargs):
        pass

class MockTaskManagerStub:
    def __init__(self, *args, **kwargs):
        pass

    def health_check(self, protobuf):
        return True

    def run_task(self, protobuf):
        return Response()

    def kill_task(self, protobuf):
        return 'kill_task'

    def get_task_status(self, protobuf):
        return 'task_status'

    def get_all_tasks(self, protobuf):
        return ['task1', 'task2']

    def get_task_log(self, protobuf):
        return 'test_log'

class MockTaskStatement:
    def __init__(self, *args, **kwargs):
        pass


class MockGoogleProtoBufEmpty:
    def __init__(self, *args, **kwargs):
        pass

    @staticmethod
    def Empty():
        pass


class MockThread:
    def __init__(self, *args, **kwargs):
        pass

    def start(self):
        pass


class MockTask:
    def __init__(self, *args, **kwargs):
        pass


class TestProcessMonitor(TestCase):

    @patch("grpc.insecure_channel", lambda x: x + "test_channel")
    @patch("threading.Thread", MockThread)
    @patch.object(experiment_scheduler.master.process_monitor, 'TaskManagerStub', MockTaskManagerStub)
    @patch.object(experiment_scheduler.master.process_monitor, 'google_dot_protobuf_dot_empty__pb2',
                  MockGoogleProtoBufEmpty)
    def setUp(self):
        self.process_monitor = ProcessMonitor('test_network', MockPipe())
        self.mock_thread_queue = {'is_healthy': False}

    def tearDown(self):
        pass

    @patch("time.sleep", side_effect=Exception)
    def test__health_check(self, mock_time_sleep):
        # when
        self.assertRaises(Exception, lambda: self.process_monitor._health_check(self.mock_thread_queue))

        # then
        self.assertEqual(self.mock_thread_queue['is_healthy'], True)

    def test_is_healthy(self):
        # when
        test_return_value = self.process_monitor.is_healthy()

        # then
        self.assertIsInstance(test_return_value, bool)

    def test__request_task_manager(self):
        # given
        command_list = \
            [("kill_task", 'test', 'test', 'test', 'test', 'test')
                , ("get_task_status", 'test', 'test', 'test', 'test', 'test')
                , ("get_all_tasks", 'test', 'test', 'test', 'test', 'test')
                , ("run_task", 'test', 'test', 'test', 'test', 'test')]
        self.process_monitor.kill_task = Mock()
        self.process_monitor.get_task_status = Mock()
        self.process_monitor.get_all_tasks = Mock()
        self.process_monitor.run_task = Mock()

        # when
        for cmd in command_list:
            self.process_monitor._request_task_manager(cmd)

        # then
        self.process_monitor.kill_task.assert_called_once()
        self.process_monitor.get_task_status.assert_called_once()
        self.process_monitor.get_all_tasks.assert_called_once()
        self.process_monitor.run_task.assert_called_once()

    @patch.object(experiment_scheduler.master.process_monitor, 'TaskStatement', MockTaskStatement)
    def test_run_task(self):
        # given
        gpu_idx = 0
        command = 'cmd'
        name = 'test'
        env = 'test_env'

        # when
        test_return_value = self.process_monitor.run_task(gpu_idx, command, name, env)

        # then
        self.assertEqual(test_return_value, 'test_id')


    def test_kill_task(self):
        # given
        task_id = 'test_id'

        # when
        test_return_value = self.process_monitor.kill_task(task_id)

        # then
        self.assertEqual(test_return_value, 'kill_task')

    def test_get_task_status(self):
        # given
        task_id = 'test_id'

        # when
        test_return_value = self.process_monitor.get_task_status(task_id)

        # then
        self.assertEqual(test_return_value, 'task_status')

    def test_get_all_tasks(self):
        # when
        test_return_value = self.process_monitor.get_all_tasks()

        # then
        self.assertEqual(test_return_value, ['task1', 'task2'])



    def test_get_task_log(self):
        # given
        task_id = 'test_id'

        # when
        test_return_value = self.process_monitor.get_task_log(task_id)

        # then
        self.assertEqual(test_return_value, 'test_log')

    @patch("time.sleep", side_effect=Exception)
    def test_start(self, mock_time_sleep):
        # given
        MockPipe.poll = Mock(return_value= True)
        MockPipe.recv = Mock(return_value= 'cmd')
        self.process_monitor._request_task_manager = Mock()

        # when
        self.assertRaises(Exception, lambda : self.process_monitor.start())

        # then
        self.process_monitor.master_pipe.poll.assert_called_once()
        self.process_monitor.master_pipe.recv.assert_called_once()
        self.process_monitor._request_task_manager.assert_called_once()






# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: task_manager.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import empty_pb2 as google_dot_protobuf_dot_empty__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x12task_manager.proto\x1a\x1bgoogle/protobuf/empty.proto\"\x1d\n\x0cServerStatus\x12\r\n\x05\x61live\x18\x01 \x01(\x08\"\x9d\x01\n\rTaskStatement\x12\x0e\n\x06gpuidx\x18\x01 \x01(\x05\x12\x0f\n\x07\x63ommand\x18\x02 \x01(\t\x12\x0c\n\x04name\x18\x03 \x01(\t\x12-\n\x08task_env\x18\x04 \x03(\x0b\x32\x1b.TaskStatement.TaskEnvEntry\x1a.\n\x0cTaskEnvEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"\x8a\x01\n\nTaskStatus\x12\x0f\n\x07task_id\x18\x01 \x01(\t\x12\"\n\x06status\x18\x02 \x01(\x0e\x32\x12.TaskStatus.Status\"G\n\x06Status\x12\x0b\n\x07RUNNING\x10\x00\x12\x08\n\x04\x44ONE\x10\x01\x12\n\n\x06KILLED\x10\x02\x12\x0c\n\x08\x41\x42NORMAL\x10\x03\x12\x0c\n\x08NOTFOUND\x10\x04\"\x17\n\x04Task\x12\x0f\n\x07task_id\x18\x01 \x01(\t\"8\n\x0e\x41llTasksStatus\x12&\n\x11task_status_array\x18\x01 \x03(\x0b\x32\x0b.TaskStatus\"\x1f\n\x07TaskLog\x12\x14\n\x0clogfile_path\x18\x01 \x01(\t2\x9c\x02\n\x0bTaskManager\x12\x37\n\x0chealth_check\x12\x16.google.protobuf.Empty\x1a\r.ServerStatus\"\x00\x12)\n\x08run_task\x12\x0e.TaskStatement\x1a\x0b.TaskStatus\"\x00\x12!\n\x0cget_task_log\x12\x05.Task\x1a\x08.TaskLog\"\x00\x12!\n\tkill_task\x12\x05.Task\x1a\x0b.TaskStatus\"\x00\x12\'\n\x0fget_task_status\x12\x05.Task\x1a\x0b.TaskStatus\"\x00\x12:\n\rget_all_tasks\x12\x16.google.protobuf.Empty\x1a\x0f.AllTasksStatus\"\x00\x42\x06\xa2\x02\x03RTGb\x06proto3')



_SERVERSTATUS = DESCRIPTOR.message_types_by_name['ServerStatus']
_TASKSTATEMENT = DESCRIPTOR.message_types_by_name['TaskStatement']
_TASKSTATEMENT_TASKENVENTRY = _TASKSTATEMENT.nested_types_by_name['TaskEnvEntry']
_TASKSTATUS = DESCRIPTOR.message_types_by_name['TaskStatus']
_TASK = DESCRIPTOR.message_types_by_name['Task']
_ALLTASKSSTATUS = DESCRIPTOR.message_types_by_name['AllTasksStatus']
_TASKLOG = DESCRIPTOR.message_types_by_name['TaskLog']
_TASKSTATUS_STATUS = _TASKSTATUS.enum_types_by_name['Status']
ServerStatus = _reflection.GeneratedProtocolMessageType('ServerStatus', (_message.Message,), {
  'DESCRIPTOR' : _SERVERSTATUS,
  '__module__' : 'task_manager_pb2'
  # @@protoc_insertion_point(class_scope:ServerStatus)
  })
_sym_db.RegisterMessage(ServerStatus)

TaskStatement = _reflection.GeneratedProtocolMessageType('TaskStatement', (_message.Message,), {

  'TaskEnvEntry' : _reflection.GeneratedProtocolMessageType('TaskEnvEntry', (_message.Message,), {
    'DESCRIPTOR' : _TASKSTATEMENT_TASKENVENTRY,
    '__module__' : 'task_manager_pb2'
    # @@protoc_insertion_point(class_scope:TaskStatement.TaskEnvEntry)
    })
  ,
  'DESCRIPTOR' : _TASKSTATEMENT,
  '__module__' : 'task_manager_pb2'
  # @@protoc_insertion_point(class_scope:TaskStatement)
  })
_sym_db.RegisterMessage(TaskStatement)
_sym_db.RegisterMessage(TaskStatement.TaskEnvEntry)

TaskStatus = _reflection.GeneratedProtocolMessageType('TaskStatus', (_message.Message,), {
  'DESCRIPTOR' : _TASKSTATUS,
  '__module__' : 'task_manager_pb2'
  # @@protoc_insertion_point(class_scope:TaskStatus)
  })
_sym_db.RegisterMessage(TaskStatus)

Task = _reflection.GeneratedProtocolMessageType('Task', (_message.Message,), {
  'DESCRIPTOR' : _TASK,
  '__module__' : 'task_manager_pb2'
  # @@protoc_insertion_point(class_scope:Task)
  })
_sym_db.RegisterMessage(Task)

AllTasksStatus = _reflection.GeneratedProtocolMessageType('AllTasksStatus', (_message.Message,), {
  'DESCRIPTOR' : _ALLTASKSSTATUS,
  '__module__' : 'task_manager_pb2'
  # @@protoc_insertion_point(class_scope:AllTasksStatus)
  })
_sym_db.RegisterMessage(AllTasksStatus)

TaskLog = _reflection.GeneratedProtocolMessageType('TaskLog', (_message.Message,), {
  'DESCRIPTOR' : _TASKLOG,
  '__module__' : 'task_manager_pb2'
  # @@protoc_insertion_point(class_scope:TaskLog)
  })
_sym_db.RegisterMessage(TaskLog)

_TASKMANAGER = DESCRIPTOR.services_by_name['TaskManager']
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\242\002\003RTG'
  _TASKSTATEMENT_TASKENVENTRY._options = None
  _TASKSTATEMENT_TASKENVENTRY._serialized_options = b'8\001'
  _SERVERSTATUS._serialized_start=51
  _SERVERSTATUS._serialized_end=80
  _TASKSTATEMENT._serialized_start=83
  _TASKSTATEMENT._serialized_end=240
  _TASKSTATEMENT_TASKENVENTRY._serialized_start=194
  _TASKSTATEMENT_TASKENVENTRY._serialized_end=240
  _TASKSTATUS._serialized_start=243
  _TASKSTATUS._serialized_end=381
  _TASKSTATUS_STATUS._serialized_start=310
  _TASKSTATUS_STATUS._serialized_end=381
  _TASK._serialized_start=383
  _TASK._serialized_end=406
  _ALLTASKSSTATUS._serialized_start=408
  _ALLTASKSSTATUS._serialized_end=464
  _TASKLOG._serialized_start=466
  _TASKLOG._serialized_end=497
  _TASKMANAGER._serialized_start=500
  _TASKMANAGER._serialized_end=784
# @@protoc_insertion_point(module_scope)



import os
from setuptools import find_packages, setup

setup(
    name="experiment-scheduler",
    version="0.1",
    author="ddanco",
    author_email="ddancoonfire@gmail.com",
    url="https://github.com/ddangcoonfire/experiment-scheduler",
    description="For whom needs easily usable multiple-experiment environment, here's experiment-scheduler",
    long_description="Experiment-Scheduler is open-source, for automating repeated experiments.\n" +
                     "In some environments like where k8s is not supported, where you can only use is ssh servers with python, repeatedly running same experiments with different parameters would be annoying and boring.\n"+
                     "By minimum settings and minimum effort, we provide distributed multi-experiment environment without affecting your already-completed server setting.\n"+
                     "Our goal is make you only concentrate on experiment by providing easily, fastly constructable experiment tool.",
    packages=find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
    ],
    python_requires='>=3.5',
    entry_points={
        "console_scripts": [
            "exs=experiment_scheduler.submitter.exs:main",
        ]
    },
)


# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc

from google.protobuf import empty_pb2 as google_dot_protobuf_dot_empty__pb2
from experiment_scheduler.task_manager.grpc_task_manager import task_manager_pb2 as task__manager__pb2


class TaskManagerStub(object):
    """Interface exported by the server.
    """

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.health_check = channel.unary_unary(
                '/TaskManager/health_check',
                request_serializer=google_dot_protobuf_dot_empty__pb2.Empty.SerializeToString,
                response_deserializer=task__manager__pb2.ServerStatus.FromString,
                )
        self.run_task = channel.unary_unary(
                '/TaskManager/run_task',
                request_serializer=task__manager__pb2.TaskStatement.SerializeToString,
                response_deserializer=task__manager__pb2.TaskStatus.FromString,
                )
        self.get_task_log = channel.unary_unary(
                '/TaskManager/get_task_log',
                request_serializer=task__manager__pb2.Task.SerializeToString,
                response_deserializer=task__manager__pb2.TaskLog.FromString,
                )
        self.kill_task = channel.unary_unary(
                '/TaskManager/kill_task',
                request_serializer=task__manager__pb2.Task.SerializeToString,
                response_deserializer=task__manager__pb2.TaskStatus.FromString,
                )
        self.get_task_status = channel.unary_unary(
                '/TaskManager/get_task_status',
                request_serializer=task__manager__pb2.Task.SerializeToString,
                response_deserializer=task__manager__pb2.TaskStatus.FromString,
                )
        self.get_all_tasks = channel.unary_unary(
                '/TaskManager/get_all_tasks',
                request_serializer=google_dot_protobuf_dot_empty__pb2.Empty.SerializeToString,
                response_deserializer=task__manager__pb2.AllTasksStatus.FromString,
                )


class TaskManagerServicer(object):
    """Interface exported by the server.
    """

    def health_check(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def run_task(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def get_task_log(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def kill_task(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def get_task_status(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def get_all_tasks(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_TaskManagerServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'health_check': grpc.unary_unary_rpc_method_handler(
                    servicer.health_check,
                    request_deserializer=google_dot_protobuf_dot_empty__pb2.Empty.FromString,
                    response_serializer=task__manager__pb2.ServerStatus.SerializeToString,
            ),
            'run_task': grpc.unary_unary_rpc_method_handler(
                    servicer.run_task,
                    request_deserializer=task__manager__pb2.TaskStatement.FromString,
                    response_serializer=task__manager__pb2.TaskStatus.SerializeToString,
            ),
            'get_task_log': grpc.unary_unary_rpc_method_handler(
                    servicer.get_task_log,
                    request_deserializer=task__manager__pb2.Task.FromString,
                    response_serializer=task__manager__pb2.TaskLog.SerializeToString,
            ),
            'kill_task': grpc.unary_unary_rpc_method_handler(
                    servicer.kill_task,
                    request_deserializer=task__manager__pb2.Task.FromString,
                    response_serializer=task__manager__pb2.TaskStatus.SerializeToString,
            ),
            'get_task_status': grpc.unary_unary_rpc_method_handler(
                    servicer.get_task_status,
                    request_deserializer=task__manager__pb2.Task.FromString,
                    response_serializer=task__manager__pb2.TaskStatus.SerializeToString,
            ),
            'get_all_tasks': grpc.unary_unary_rpc_method_handler(
                    servicer.get_all_tasks,
                    request_deserializer=google_dot_protobuf_dot_empty__pb2.Empty.FromString,
                    response_serializer=task__manager__pb2.AllTasksStatus.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'TaskManager', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))


 # This class is part of an EXPERIMENTAL API.
class TaskManager(object):
    """Interface exported by the server.
    """

    @staticmethod
    def health_check(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/TaskManager/health_check',
            google_dot_protobuf_dot_empty__pb2.Empty.SerializeToString,
            task__manager__pb2.ServerStatus.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def run_task(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/TaskManager/run_task',
            task__manager__pb2.TaskStatement.SerializeToString,
            task__manager__pb2.TaskStatus.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def get_task_log(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/TaskManager/get_task_log',
            task__manager__pb2.Task.SerializeToString,
            task__manager__pb2.TaskLog.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def kill_task(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/TaskManager/kill_task',
            task__manager__pb2.Task.SerializeToString,
            task__manager__pb2.TaskStatus.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def get_task_status(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/TaskManager/get_task_status',
            task__manager__pb2.Task.SerializeToString,
            task__manager__pb2.TaskStatus.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def get_all_tasks(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/TaskManager/get_all_tasks',
            google_dot_protobuf_dot_empty__pb2.Empty.SerializeToString,
            task__manager__pb2.AllTasksStatus.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)
